///////
     //
     //  MOBOTIX EventStream Client SDK 
     //
     //  See LICENSE_SDK included in the SDK package
     //
     //  Copyright (c) 2005 - 2016, MOBOTIX AG
     //  All rights reserved.
     //
///////
    

#ifndef M1IMAGE_JSIMAGE_SRC_MXPEG_SDK_API_HPP_
#define M1IMAGE_JSIMAGE_SRC_MXPEG_SDK_API_HPP_


#include "MxPEG_SinkVideo.hpp"
#include "MxPEG_SinkAudio.hpp"
#include "MxPEG_AuthorizationHandler.hpp"
#include "MxPEG_EventNotificationListener.hpp"

namespace ie { namespace MxPEG {

class MxPEG_SDK
{
public:
   MxPEG_SDK (MxPEG_SinkAudio *audioSink, MxPEG_SinkVideo *sinkVideo, MxPEG_AuthorizationHandler *authHandler, MxPEG_EventNotificationListener *notificationListener,
			  MxPEG_ImageMode decodingMode);

   virtual
   ~MxPEG_SDK ();

   /*
    * Main loop of the SDK package, returns after one "iteration". Needs to be called repeatedly from
    * the applications main loop.
    */
   void loop();

   /*
    * Setup the library internally.
    *
    * @url URL of the EventStream service. Typically this should look like this:
    *
    * http://<cam-ip>:<port>/control/eventstream.jpg
    *
    * Return er_Success on success
    */
   MxPEG_ReturnCode stream_setup(const char *url);


   /*
    * Establish a connection with the camera. (Implicitly called by stream_setup)
    *
    * Returns er_Success if the connection could be established
    */
   MxPEG_ReturnCode stream_connect();

   /*
    * Close the connection.
    *
    * @reconnect: Immediately try to connect to the service after closing the connection.
    *
    * Returns er_Success, or er_* error code.
    */
   MxPEG_ReturnCode stream_close(bool reconnect = false);


   /*
    * Get a new id to be used for when calling queue_command()
    *
    * Return: On success positive integer number to be used as id for queue_command(), on error -1
    */
   int get_command_id();

   /*
    * Send a low-level message/command to the camera. Using this function any command available from
    * the EventStream service can be invoked.
    *
    * @id The id used to tag the command, the camera will use this id when sending replies. Use get_command_id to get a valid, unique id number
    * @ctrlCmd string holding a JSON message to be sent to the camera.
    *
    * Returns er_Success, or er_* error code.
    *
    * Please note: The return type "split" for generic commands is not yet supported in this library.
    */
   MxPEG_ReturnCode queue_command(int id, const char * ctrlCmd);

   /*
    * Enable debug logging in the SDK. Log messages will be printed to stdout. See MxPEG_Logger.hpp for supported values for
    * target and mask.
    *
    * Mask ids of the modules in the SDK:
    *
    *	SYSTEM           1
	*	NET              2
	*	VIDEO            4
	*	AUDIO            8
	*	C_API            16
	*	DECODER          32
	*	EVENT            64
    *
    * @mask Sum of all log mask ids.
    *
    * Returns er_Success, or er_* error code.
    */
   MxPEG_ReturnCode set_log_mask(int mask);

   /*
    * Audio return channel. Play back audio on the camera (only models with integrated speaker).
    *
    * Start a new audio return session.
    *
    * Returns er_Success, or er_* error code.
    */
   MxPEG_ReturnCode sendAudioStart();

   /*
    * Stop the audio session.
    *
    * Returns er_Success, or er_* error code.
    */
   MxPEG_ReturnCode sendAudioStop();

   /*
    * Send audio raw data to the camera.
    *
    * @format: the format of the audio data passed to this function (right now only PCM16LE, 16kHz, 1 channel is supported)
    * @rawAudio: Buffer holding raw audio data as specified by format
    * @rawAudioSize: Size of the audio buffer
    *
    * Returns er_Success, or er_* error code.
    */
   MxPEG_ReturnCode sendAudio(MxPEG_AudioFormat format, uint8_t *rawAudio, size_t rawAudioSize);

   /*
    * Activate video for the current session. Without this the camera will not send any video frames.
    *
    * @active "true" activates video transmission for the current session, "false" disables it.
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    */
   uint32_t setVideoActive(bool active);

   /*
    * Set the encoding mode for the video frames.
    *
    * @mode: Video encoding mode for this session. Supported modes are MJPEG, MxPEG (default)
    *
    * Return: On success the command id associated with the request, 0 otherwise
    */
   uint32_t setVideoMode(MxPEG_VideoMode mode);

   /*
    * Set the video resolution for the stream.
    *
    * @width Width of the video stream
    * @height Height of the video stream
    * @configmode: true to allow global reconfiguration of the camera
    *
    * If configmode is "false" the size change is limited to the current session, max resolution is the
    * currently configured "Image Size".
    *
    * If configmode is "true" this will change the image size globally.
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t setResolution(uint32_t width, uint32_t height, bool configMode = false);


   /*
    * Set fps for the stream.
    *
    * @fps Frame rate at which to generate the video stream
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t setVideoFps(float fps);


   /*
    * Set the sensor for the video stream.
    *
    * @sensor: Select the active sensor. Valid options depend on the camera model and equipped sensors.
    *
    * Note: This option automatically implies configmode = true!
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t setVideoSensor(MxPEG_Sensor sensor);


   /*
    * Set the display_mode for the video stream.
    *

    * @displayMode: string specifying the display mode for the video stream. Valid options are:
    *
    * 	"normal": Shows the full image of the selected image sensor. For hemispheric lenses, the original distorted full image is displayed.
    * 	"lenscorr": Shows a corrected image from non-hemispheric lenses in 4:3 format.
    *	"surround": Shows four images with independent views.
    *	"panorama": Shows a panorama image in 8:3 format.
    *	"pano_dbl": Shows two panorama images in 8:3 format.
    *	"pano_focus": Shows one panorama image in 8:3 format and two 4:3 images below.
    *	"pip": Shows a small preview image of the other sensor.
    *	"piz": Shows a small overview image in the zoomed image.
    *	"eip": Shows the last event image in the live image.
    *
    * @configmode: true to allow global reconfiguration of the camera
    *
    * Calling this function with configmode "false" will only have an effect if displaymode is globally
    * set to "Full Image".
    *
    * Note: Depending on your camera model, some of these features may not be available!
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t setVideoDisplayMode(std::string displayMode, bool configMode = true);

   /*
    * Enable/Disable audio for the current session.
    *
    * @active "true" to activate audio, "false" otherwise
    *
    * setAudioActive(true) is equal to setAudioMode("pcm16")
    *
    * setAudioActive(false) is equal to setAudioMode("off")
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t setAudioActive(bool active);

   /*
    * Configures the audio output mode for the current session. Supported modes are
    *
    * @mode String that specifies the type of audio data used in this session
    *
    * "pcm16": Send audio data as PCM16 little endian
    * "alaw": Audio data using aLaw encoding
    *
    * Please note: This affects only Audio encoding for the live video stream! Recordings will
    * always include aLaw encoded audio!
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t setAudioMode(std::string mode);

   /*
    * Start streaming the live feed from the cameras currently active sensor
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t startLive();

   /*
    * Start playback of the recording at startTime
    *
    * @startTime: Timestamp from which to start playback. If no recording is available from
    * the exact time, the next recoding after that point in time is started.
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    */
   uint32_t play(struct timeval startTime);

   /*
    * Start playback of the recording at the current player position
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t play();

   /*
    * Pauses the current playback session.
    *
    * Returns On success: the command id associated with the request, 0 otherwise
    *
    */
   uint32_t pause();

   /*
    * Sets the playback rate of the current playback session.
    *
    * @rate Specifies the playback rate. Use values greater 1 to implement fast
    * forward and values between 0 and 1 to implement "slow motion". Use negative values to implement
    * (fast) reverse playback. Setting the rate to 0 causes the stream to pause.
    *
    * Note: Audio is only available if the playback rate is set to 1.
    *
    */
   uint32_t setPlaybackRate(float rate);

   /*
    * Position the player on the HEAD (most recent) event.
    */
   uint32_t playerPositionHead();

   /*
    * Position the player on the TAIL (oldest available) event.
    */
   uint32_t playerPositionTail();

   /*
    * Move the player position from the current position to the next (more recent) event.
    */
   uint32_t playerPositionNext();

   /*
    * Move the player position from the current position to the previous (older) event.
    */
   uint32_t playerPositionPrev();

   /*
    * Zoom in/out either relatively or set zoom value absolute.
    *
    * @x: x-coordinate of region of interest (origin top-left)
    * @y: y-coordinate of region of interest (origin top-left)
    * @zoom Choose zoom values from [1000...8000] to set zoom from 1x up to 8x
    * @configmode: true to allow global reconfiguration of the camera
    *
    * If the display mode is configured to show multiple windows in one stream (e.g. Surround) the
    * region of interest is used to determine the sub-window in which to
    * execute the zoom operation.
    *
    * If configMode is "false" the zoom will only use the current picture stream as input. If it is set to
    * "true" the zoom operation might also reconfigure the camera to create higher resolution input pictures.
    *
    * Please note: Zooming in sub-windows always requires configMode=true.
    *
    * Please further note: The camera does not digitally enlarge the picture that is generated by the sensor. If
    * the zoom operation reaches that limit, the camera will start cropping the video to the region the would
    * be the input of such a digital zoom operation. But it is up to the client to actually enlarge the cropped
    * video.
    *
    */
   uint32_t ptzSetZoom(uint32_t x, uint32_t y, int32_t zoom, bool zoomRelative = false, bool configMode = false);

   /*
    * Set image center to image coordinates x,y (origin top left corner)
    *
    * @x: x-coordinate of image center (origin top-left)
    * @y: y-coordinate of image center (origin top-left)
    */
   uint32_t ptzSetCenter(uint32_t x, uint32_t y, bool configMode  = false);

   /*
    * Provides access the to current streams M1IMG data. The requested data field is specified
    * by section and property name. If the field is not available NULL is returned.
    *
    * The caller needs to release the string.
    */
   char * stream_info_m1img(char * section, char * property);

   /*
    * Returns the timestamp of the most recent event stored on the camera (extracted from the MXF headers).
    *
    * This function only returns valid data while the live video stream is active.
    */
   uint32_t stream_info_last_event();

private:
   class MxPEG_Main * m_main;

};

}}

#endif /* M1IMAGE_JSIMAGE_SRC_MXPEG_SDK_API_HPP_ */
